package java.util;

import java.io.Serializable;

public abstract class AbstractMap<K, V> implements Map<K, V>
{
    transient Set<K> keySet;
    transient Collection<V> values;
    
    protected AbstractMap() {
    }
    
    @Override
    public int size() {
        return this.entrySet().size();
    }
    
    @Override
    public boolean isEmpty() {
        return this.size() == 0;
    }
    
    @Override
    public boolean containsValue(Object o) {
        Iterator<Map.Entry<K, V>> iterator = this.entrySet().iterator();
        if (o == null) {
            while (iterator.hasNext()) {
                if (((Map.Entry)iterator.next()).getValue() == null) {
                    return true;
                }
                continue;
            }
        }
        else {
            while (iterator.hasNext()) {
                if (o.equals(((Map.Entry)iterator.next()).getValue())) {
                    return true;
                }
                continue;
            }
        }
        return false;
    }
    
    @Override
    public boolean containsKey(Object o) {
        Iterator<Map.Entry<K, V>> iterator = this.entrySet().iterator();
        if (o == null) {
            while (iterator.hasNext()) {
                if (((Map.Entry)iterator.next()).getKey() == null) {
                    return true;
                }
                continue;
            }
        }
        else {
            while (iterator.hasNext()) {
                if (o.equals(((Map.Entry)iterator.next()).getKey())) {
                    return true;
                }
                continue;
            }
        }
        return false;
    }
    
    @Override
    public V get(Object o) {
        Iterator<Map.Entry<K, V>> iterator = this.entrySet().iterator();
        if (o == null) {
            while (iterator.hasNext()) {
                Map.Entry<Object, Object> entry2 = (Map.Entry)iterator.next();
                if (entry2.getKey() == null) {
                    return entry2.getValue();
                }
                continue;
            }
        }
        else {
            while (iterator.hasNext()) {
                Map.Entry<Object, Object> entry = (Map.Entry)iterator.next();
                if (o.equals(entry.getKey())) {
                    return entry.getValue();
                }
                continue;
            }
        }
        return null;
    }
    
    @Override
    public V put(K k, V v) {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public V remove(Object o) {
        Iterator<Map.Entry<K, V>> iterator = this.entrySet().iterator();
        Map.Entry<Object, Object> entry = null;
        if (o == null) {
            while (entry == null && iterator.hasNext()) {
                Map.Entry<Object, Object> entry3 = (Map.Entry)iterator.next();
                if (entry3.getKey() == null) {
                    entry = entry3;
                }
                else {
                    continue;
                }
            }
        }
        else {
            while (entry == null && iterator.hasNext()) {
                Map.Entry<Object, Object> entry2 = (Map.Entry)iterator.next();
                if (o.equals(entry2.getKey())) {
                    entry = entry2;
                }
                else {
                    continue;
                }
            }
        }
        Object value = null;
        if (entry != null) {
            value = entry.getValue();
            iterator.remove();
            return value;
        }
        return value;
    }
    
    @Override
    public void putAll(Map<? extends K, ? extends V> map) {
        for (Map.Entry entry : map.entrySet()) {
            this.put(entry.getKey(), entry.getValue());
        }
    }
    
    @Override
    public void clear() {
        this.entrySet().clear();
    }
    
    @Override
    public Set<K> keySet() {
        Set<Object> keySet = this.keySet;
        if (keySet == null) {
            keySet = new AbstractSet<K>() {
                @Override
                public Iterator<K> iterator() {
                    return new Iterator<K>() {
                        private Iterator<Map.Entry<K, V>> i = AbstractMap.this.entrySet().iterator();
                        
                        @Override
                        public boolean hasNext() {
                            return this.i.hasNext();
                        }
                        
                        @Override
                        public K next() {
                            return ((Map.Entry)this.i.next()).getKey();
                        }
                        
                        @Override
                        public void remove() {
                            this.i.remove();
                        }
                    };
                }
                
                @Override
                public int size() {
                    return AbstractMap.this.size();
                }
                
                @Override
                public boolean isEmpty() {
                    return AbstractMap.this.isEmpty();
                }
                
                @Override
                public void clear() {
                    AbstractMap.this.clear();
                }
                
                @Override
                public boolean contains(Object o) {
                    return AbstractMap.this.containsKey(o);
                }
            };
            this.keySet = keySet;
            return keySet;
        }
        return keySet;
    }
    
    @Override
    public Collection<V> values() {
        Collection<Object> values = this.values;
        if (values == null) {
            values = new AbstractCollection<V>() {
                @Override
                public Iterator<V> iterator() {
                    return new Iterator<V>() {
                        private Iterator<Map.Entry<K, V>> i = AbstractMap.this.entrySet().iterator();
                        
                        @Override
                        public boolean hasNext() {
                            return this.i.hasNext();
                        }
                        
                        @Override
                        public V next() {
                            return ((Map.Entry)this.i.next()).getValue();
                        }
                        
                        @Override
                        public void remove() {
                            this.i.remove();
                        }
                    };
                }
                
                @Override
                public int size() {
                    return AbstractMap.this.size();
                }
                
                @Override
                public boolean isEmpty() {
                    return AbstractMap.this.isEmpty();
                }
                
                @Override
                public void clear() {
                    AbstractMap.this.clear();
                }
                
                @Override
                public boolean contains(Object o) {
                    return AbstractMap.this.containsValue(o);
                }
            };
            this.values = values;
            return values;
        }
        return values;
    }
    
    @Override
    public abstract Set<Map.Entry<K, V>> entrySet();
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof Map)) {
            return false;
        }
        Map<Object, Object> map = (Map)o;
        if (map.size() != this.size()) {
            return false;
        }
        try {
            for (Map.Entry entry : this.entrySet()) {
                Object key = entry.getKey();
                Object value = entry.getValue();
                if (value == null) {
                    if (map.get(key) != null || !map.containsKey(key)) {
                        return false;
                    }
                    continue;
                }
                else {
                    if (!value.equals(map.get(key))) {
                        return false;
                    }
                    continue;
                }
            }
        }
        catch (ClassCastException ex) {
            return false;
        }
        catch (NullPointerException ex2) {
            return false;
        }
        return true;
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        Iterator<Map.Entry<K, V>> iterator = this.entrySet().iterator();
        while (iterator.hasNext()) {
            n += ((Map.Entry)iterator.next()).hashCode();
        }
        return n;
    }
    
    @Override
    public String toString() {
        Iterator<Map.Entry<K, V>> iterator = this.entrySet().iterator();
        if (!iterator.hasNext()) {
            return "{}";
        }
        StringBuilder sb = new StringBuilder();
        sb.append('{');
        while (true) {
            Map.Entry<Object, Object> entry = (Map.Entry)iterator.next();
            Object key = entry.getKey();
            Object value = entry.getValue();
            sb.append((key == this) ? "(this Map)" : key);
            sb.append('=');
            sb.append((value == this) ? "(this Map)" : value);
            if (!iterator.hasNext()) {
                break;
            }
            sb.append(',').append(' ');
        }
        return sb.append('}').toString();
    }
    
    @Override
    protected Object clone() throws CloneNotSupportedException {
        AbstractMap<Object, Object> abstractMap = (AbstractMap)super.clone();
        abstractMap.keySet = null;
        abstractMap.values = null;
        return abstractMap;
    }
    
    private static boolean eq(Object o, Object obj) {
        return (o == null) ? (obj == null) : o.equals(obj);
    }
    
    static boolean access$000(Object o, Object o2) {
        return eq(o, o2);
    }
    
    public static class SimpleEntry<K, V> implements Map.Entry<K, V>, Serializable
    {
        private static final long serialVersionUID = -8499721149061103585L;
        private final K key;
        private V value;
        
        public SimpleEntry(K key, V value) {
            this.key = key;
            this.value = value;
        }
        
        public SimpleEntry(Map.Entry<? extends K, ? extends V> entry) {
            this.key = entry.getKey();
            this.value = entry.getValue();
        }
        
        @Override
        public K getKey() {
            return this.key;
        }
        
        @Override
        public V getValue() {
            return this.value;
        }
        
        @Override
        public V setValue(V value) {
            Object value2 = this.value;
            this.value = value;
            return value2;
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            return AbstractMap.access$000(this.key, entry.getKey()) && AbstractMap.access$000(this.value, entry.getValue());
        }
        
        @Override
        public int hashCode() {
            return ((this.key == null) ? 0 : this.key.hashCode()) ^ ((this.value == null) ? 0 : this.value.hashCode());
        }
        
        @Override
        public String toString() {
            return this.key + "=" + this.value;
        }
    }
    
    public static class SimpleImmutableEntry<K, V> implements Map.Entry<K, V>, Serializable
    {
        private static final long serialVersionUID = 7138329143949025153L;
        private final K key;
        private final V value;
        
        public SimpleImmutableEntry(K key, V value) {
            this.key = key;
            this.value = value;
        }
        
        public SimpleImmutableEntry(Map.Entry<? extends K, ? extends V> entry) {
            this.key = entry.getKey();
            this.value = entry.getValue();
        }
        
        @Override
        public K getKey() {
            return this.key;
        }
        
        @Override
        public V getValue() {
            return this.value;
        }
        
        @Override
        public V setValue(V v) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            return AbstractMap.access$000(this.key, entry.getKey()) && AbstractMap.access$000(this.value, entry.getValue());
        }
        
        @Override
        public int hashCode() {
            return ((this.key == null) ? 0 : this.key.hashCode()) ^ ((this.value == null) ? 0 : this.value.hashCode());
        }
        
        @Override
        public String toString() {
            return this.key + "=" + this.value;
        }
    }
}
